from __future__ import unicode_literals

import logging

from django.core.management.base import BaseCommand

from django_keycloak.models import Client

import django_keycloak.services.uma

logger = logging.getLogger(__name__)


def client(client_id):
    try:
        return Client.objects.get(client_id=client_id)
    except Client.DoesNotExist:
        raise TypeError('Client does not exist')


class Command(BaseCommand):

    def add_arguments(self, parser):
        parser.add_argument('--client', type=client, required=False)

    def handle(self, *args, **options):
        client = options.get('client')

        if client:
            django_keycloak.services.uma.synchronize_client(client=client)
        else:
            for client in Client.objects.all():
                django_keycloak.services.uma.synchronize_client(client=client)



from django.contrib.auth import HASH_SESSION_KEY, BACKEND_SESSION_KEY, \
    _get_backends
from django.contrib.auth.models import AnonymousUser
from django.middleware.csrf import rotate_token
from django.utils import timezone

import django_keycloak.services.oidc_profile


# Using a different session key than the standard django.contrib.auth to
# make sure there is no cross-referencing between UserModel and RemoteUserModel
REMOTE_SESSION_KEY = '_auth_remote_user_id'


def _get_user_session_key(request):
    return str(request.session[REMOTE_SESSION_KEY])


def get_remote_user(request):
    """

    :param request:
    :return:
    """
    sub = request.session.get(REMOTE_SESSION_KEY)

    user = None

    OpenIdConnectProfile = django_keycloak.services.oidc_profile\
        .get_openid_connect_profile_model()

    try:
        oidc_profile = OpenIdConnectProfile.objects.get(
            realm=request.realm, sub=sub)
    except OpenIdConnectProfile.DoesNotExist:
        pass
    else:
        if oidc_profile.refresh_expires_before > timezone.now():
            user = oidc_profile.user

    return user or AnonymousUser()


def remote_user_login(request, user, backend=None):
    """
    Creates a session for the user.
    Based on the login function django.contrib.auth.login but uses a slightly
    different approach since the user is not backed by a database model.
    :param request:
    :param user:
    :param backend:
    :return:
    """
    session_auth_hash = ''
    if user is None:
        user = request.user

    if REMOTE_SESSION_KEY in request.session:
        if _get_user_session_key(request) != user.identifier:
            request.session.flush()
    else:
        request.session.cycle_key()

    try:
        backend = backend or user.backend
    except AttributeError:
        backends = _get_backends(return_tuples=True)
        if len(backends) == 1:
            _, backend = backends[0]
        else:
            raise ValueError(
                'You have multiple authentication backends configured and '
                'therefore must provide the `backend` argument or set the '
                '`backend` attribute on the user.'
            )

    if not hasattr(user, 'identifier'):
        raise ValueError(
            'The user does not have an identifier or the identifier is empty.'
        )

    request.session[REMOTE_SESSION_KEY] = user.identifier
    request.session[BACKEND_SESSION_KEY] = backend
    request.session[HASH_SESSION_KEY] = session_auth_hash
    if hasattr(request, 'user'):
        request.user = user
    rotate_token(request)



import re

from django.conf import settings
from django.contrib.auth import authenticate
from django.contrib.auth.models import AnonymousUser
from django.core.exceptions import ObjectDoesNotExist
from django.utils.deprecation import MiddlewareMixin
from django.utils.functional import SimpleLazyObject

from django_keycloak.models import Realm
from django_keycloak.auth import get_remote_user
from django_keycloak.response import HttpResponseNotAuthorized


def get_realm(request):
    if not hasattr(request, '_cached_realm'):
        request._cached_realm = Realm.objects.first()
    return request._cached_realm


def get_user(request, origin_user):
    # Check for the user as set by
    # django.contrib.auth.middleware.AuthenticationMiddleware
    if not isinstance(origin_user, AnonymousUser):
        return origin_user

    if not hasattr(request, '_cached_user'):
        request._cached_user = get_remote_user(request)
    return request._cached_user


class BaseKeycloakMiddleware(MiddlewareMixin):

    set_session_state_cookie = True

    def process_request(self, request):
        """
        Adds Realm to request.
        :param request: django request
        """
        request.realm = SimpleLazyObject(lambda: get_realm(request))

    def process_response(self, request, response):

        if self.set_session_state_cookie:
            return self.set_session_state_cookie_(request, response)

        return response

    def set_session_state_cookie_(self, request, response):

        if not request.user.is_authenticated:
            return response

        try:
            oidc_profile = request.user.oidc_profile
        except ObjectDoesNotExist:
            return response

        jwt = oidc_profile.jwt
        if not jwt:
            return response

        cookie_name = getattr(settings, 'KEYCLOAK_SESSION_STATE_COOKIE_NAME',
                              'session_state')

        # Set a browser readable cookie which expires when the refresh token
        # expires.
        response.set_cookie(
            cookie_name, value=jwt['session_state'],
            expires=oidc_profile.refresh_expires_before,
            httponly=False
        )

        return response


class KeycloakStatelessBearerAuthenticationMiddleware(BaseKeycloakMiddleware):

    set_session_state_cookie = False
    header_key = "HTTP_AUTHORIZATION"

    def process_request(self, request):
        """
        Forces authentication on all requests except the URL's configured in
        the exempt setting.
        """
        super(KeycloakStatelessBearerAuthenticationMiddleware, self)\
            .process_request(request=request)

        if hasattr(settings, 'KEYCLOAK_BEARER_AUTHENTICATION_EXEMPT_PATHS'):
            path = request.path_info.lstrip('/')

            if any(re.match(m, path) for m in
                   settings.KEYCLOAK_BEARER_AUTHENTICATION_EXEMPT_PATHS):
                return

        if self.header_key not in request.META:
            return HttpResponseNotAuthorized(
                attributes={'realm': request.realm.name})

        user = authenticate(
            request=request,
            access_token=request.META[self.header_key].split(' ')[1]
        )

        if user is None:
            return HttpResponseNotAuthorized(
                attributes={'realm': request.realm.name})
        else:
            request.user = user


class RemoteUserAuthenticationMiddleware(MiddlewareMixin):
    set_session_state_cookie = False

    def process_request(self, request):
        """
        Adds user to the request when authorized user is found in the session
        :param django.http.request.HttpRequest request: django request
        """
        origin_user = getattr(request, 'user', None)

        request.user = SimpleLazyObject(lambda: get_user(
            request,
            origin_user=origin_user
        ))



import factory

from django.contrib.auth import get_user_model

from django_keycloak.models import (
    Client,
    OpenIdConnectProfile,
    Realm,
    Server
)


class UserFactory(factory.DjangoModelFactory):

    class Meta(object):
        model = get_user_model()

    username = factory.Faker('user_name')


class ServerFactory(factory.DjangoModelFactory):

    class Meta(object):
        model = Server

    url = factory.Faker('url', schemes=['https'])


class RealmFactory(factory.DjangoModelFactory):

    class Meta(object):
        model = Realm

    server = factory.SubFactory(ServerFactory)

    name = factory.Faker('slug')

    _certs = ''
    _well_known_oidc = '{}'

    client = factory.RelatedFactory('django_keycloak.factories.ClientFactory',
                                    'realm')


class OpenIdConnectProfileFactory(factory.DjangoModelFactory):

    class Meta(object):
        model = OpenIdConnectProfile

    sub = factory.Faker('uuid4')
    realm = factory.SubFactory(RealmFactory)
    user = factory.SubFactory(UserFactory)


class ClientFactory(factory.DjangoModelFactory):

    class Meta(object):
        model = Client

    realm = factory.SubFactory(RealmFactory, client=None)
    service_account_profile = factory.SubFactory(
        OpenIdConnectProfileFactory,
        realm=factory.SelfAttribute('..realm')
    )

    client_id = factory.Faker('slug')
    secret = factory.Faker('uuid4')



import mock

from datetime import datetime

from django.test import TestCase
from keycloak.openid_connect import KeycloakOpenidConnect

from django_keycloak.factories import ClientFactory, \
    OpenIdConnectProfileFactory, UserFactory
from django_keycloak.tests.mixins import MockTestCaseMixin

import django_keycloak.services.oidc_profile


class ServicesOpenIDProfileGetOrCreateFromIdTokenTestCase(
        MockTestCaseMixin, TestCase):

    def setUp(self):
        self.client = ClientFactory(
            realm___certs='{}',
            realm___well_known_oidc='{"issuer": "https://issuer"}'
        )
        self.client.openid_api_client = mock.MagicMock(
            spec_set=KeycloakOpenidConnect)
        self.client.openid_api_client.well_known = {
            'id_token_signing_alg_values_supported': ['signing-alg']
        }
        self.client.openid_api_client.decode_token.return_value = {
            'sub': 'some-sub',
            'email': 'test@example.com',
            'given_name': 'Some given name',
            'family_name': 'Some family name'
        }

    def test_create_with_new_user_new_profile(self):
        """
        Case: oidc profile is requested based on a provided id token.
        The user and profile do not exist yet.
        Expected: oidc profile and user are created with information from
        the id token.
        """
        profile = django_keycloak.services.oidc_profile. \
            get_or_create_from_id_token(
                client=self.client, id_token='some-id-token'
            )

        self.client.openid_api_client.decode_token.assert_called_with(
            token='some-id-token',
            key=dict(),
            algorithms=['signing-alg'],
            issuer='https://issuer'
        )

        self.assertEqual(profile.sub, 'some-sub')
        self.assertEqual(profile.user.username, 'some-sub')
        self.assertEqual(profile.user.email, 'test@example.com')
        self.assertEqual(profile.user.first_name, 'Some given name')
        self.assertEqual(profile.user.last_name, 'Some family name')

    def test_update_with_existing_profile_new_user(self):
        """
        Case: oidc profile is requested based on a provided id token.
        The profile exists, but the user doesn't.
        Expected: oidc user is created with information from the id token
        and linked to the profile.
        """
        existing_profile = OpenIdConnectProfileFactory(
            access_token='access-token',
            expires_before=datetime(2018, 3, 5, 1, 0, 0),
            refresh_token='refresh-token',
            sub='some-sub'
        )

        profile = django_keycloak.services.oidc_profile. \
            get_or_create_from_id_token(
                client=self.client, id_token='some-id-token'
            )

        self.client.openid_api_client.decode_token.assert_called_with(
            token='some-id-token',
            key=dict(),
            algorithms=['signing-alg'],
            issuer='https://issuer'
        )

        self.assertEqual(profile.sub, 'some-sub')
        self.assertEqual(profile.pk, existing_profile.pk)
        self.assertEqual(profile.user.username, 'some-sub')
        self.assertEqual(profile.user.email, 'test@example.com')
        self.assertEqual(profile.user.first_name, 'Some given name')
        self.assertEqual(profile.user.last_name, 'Some family name')

    def test_create_with_existing_user_new_profile(self):
        """
        Case: oidc profile is requested based on a provided id token.
        The user exists, but the profile doesn't.
        Expected: oidc profile is created and user is linked to the profile.
        """
        existing_user = UserFactory(
            username='some-sub'
        )

        profile = django_keycloak.services.oidc_profile.\
            get_or_create_from_id_token(
                client=self.client, id_token='some-id-token'
            )

        self.client.openid_api_client.decode_token.assert_called_with(
            token='some-id-token',
            key=dict(),
            algorithms=['signing-alg'],
            issuer='https://issuer'
        )

        self.assertEqual(profile.sub, 'some-sub')
        self.assertEqual(profile.user.pk, existing_user.pk)
        self.assertEqual(profile.user.username, 'some-sub')
        self.assertEqual(profile.user.email, 'test@example.com')
        self.assertEqual(profile.user.first_name, 'Some given name')
        self.assertEqual(profile.user.last_name, 'Some family name')

    def test_create_with_existing_user_existing_profile(self):
        """
        Case: oidc profile is requested based on a provided id token.
        The user and profile already exist.
        Expected: existing oidc profile is returned with existing user linked
        to it.
        """
        existing_user = UserFactory(
            username='some-sub'
        )

        existing_profile = OpenIdConnectProfileFactory(
            access_token='access-token',
            expires_before=datetime(2018, 3, 5, 1, 0, 0),
            refresh_token='refresh-token',
            sub='some-sub'
        )

        profile = django_keycloak.services.oidc_profile.\
            get_or_create_from_id_token(
                client=self.client, id_token='some-id-token'
            )

        self.client.openid_api_client.decode_token.assert_called_with(
            token='some-id-token',
            key=dict(),
            algorithms=['signing-alg'],
            issuer='https://issuer'
        )

        self.assertEqual(profile.pk, existing_profile.pk)
        self.assertEqual(profile.sub, 'some-sub')
        self.assertEqual(profile.user.pk, existing_user.pk)
        self.assertEqual(profile.user.username, 'some-sub')
        self.assertEqual(profile.user.email, 'test@example.com')
        self.assertEqual(profile.user.first_name, 'Some given name')
        self.assertEqual(profile.user.last_name, 'Some family name')



from django.contrib import admin, messages
from keycloak.exceptions import KeycloakClientError
from requests.exceptions import HTTPError

from django_keycloak.models import (
    Client,
    OpenIdConnectProfile,
    RemoteClient
)
import django_keycloak.services.permissions
import django_keycloak.services.realm
import django_keycloak.services.uma


def refresh_open_id_connect_well_known(modeladmin, request, queryset):
    for realm in queryset:
        django_keycloak.services.realm.refresh_well_known_oidc(realm=realm)
    modeladmin.message_user(
        request=request,
        message='OpenID Connect .well-known refreshed',
        level=messages.SUCCESS
    )


refresh_open_id_connect_well_known.short_description = 'Refresh OpenID ' \
                                                       'Connect .well-known'


def refresh_certs(modeladmin, request, queryset):
    for realm in queryset:
        django_keycloak.services.realm.refresh_certs(realm=realm)
    modeladmin.message_user(
        request=request,
        message='Certificates refreshed',
        level=messages.SUCCESS
    )


refresh_certs.short_description = 'Refresh Certificates'


def clear_client_tokens(modeladmin, request, queryset):
    OpenIdConnectProfile.objects.filter(realm__in=queryset).update(
        access_token=None,
        expires_before=None,
        refresh_token=None,
        refresh_expires_before=None
    )
    modeladmin.message_user(
        request=request,
        message='Tokens cleared',
        level=messages.SUCCESS
    )


clear_client_tokens.short_description = 'Clear client tokens'


def synchronize_permissions(modeladmin, request, queryset):
    for realm in queryset:
        try:
            django_keycloak.services.permissions.synchronize(
                client=realm.client)
        except HTTPError as e:
            if e.response.status_code == 403:
                modeladmin.message_user(
                    request=request,
                    message='Forbidden for {}. Does the client\'s service '
                            'account has the "keycloak_client" role?'.format(
                                realm.name
                            ),
                    level=messages.ERROR
                )
                return
            else:
                raise
    modeladmin.message_user(
        request=request,
        message='Permissions synchronized',
        level=messages.SUCCESS
    )


synchronize_permissions.short_description = 'Synchronize permissions'


def synchronize_resources(modeladmin, request, queryset):
    for realm in queryset:
        try:
            django_keycloak.services.uma.synchronize_client(
                client=realm.client)
        except KeycloakClientError as e:
            if e.original_exc.response.status_code == 400:
                modeladmin.message_user(
                    request=request,
                    message='Forbidden for {}. Is "Remote Resource '
                            'Management"  enabled for the related client?'
                            .format(
                                realm.name
                            ),
                    level=messages.ERROR
                )
                return
            else:
                raise
    modeladmin.message_user(
        request=request,
        message='Resources synchronized',
        level=messages.SUCCESS
    )


synchronize_resources.short_description = 'Synchronize models as Keycloak ' \
                                          'resources'


class ClientAdmin(admin.TabularInline):

    model = Client

    fields = ('client_id', 'secret')


class RemoteClientAdmin(admin.TabularInline):

    model = RemoteClient

    extra = 1

    fields = ('name',)


class RealmAdmin(admin.ModelAdmin):

    inlines = [ClientAdmin, RemoteClientAdmin]

    actions = [
        refresh_open_id_connect_well_known,
        refresh_certs,
        clear_client_tokens,
        synchronize_permissions,
        synchronize_resources
    ]

    fieldsets = (
        (None, {
            'fields': ('name',)
        }),
        ('Location', {
            'fields': ('server', '_well_known_oidc',)
        })

    )

    readonly_fields = ('_well_known_oidc',)



import mock

from django.test import TestCase

from django_keycloak.factories import RealmFactory
from django_keycloak.tests.mixins import MockTestCaseMixin

import django_keycloak.services.realm


class ServicesRealmRefreshWellKnownOIDCTestCase(
        MockTestCaseMixin, TestCase):

    def setUp(self):
        self.realm = RealmFactory(
            name='test-realm',
            _well_known_oidc='empty'
        )

        keycloak_oidc_mock = mock.MagicMock()
        keycloak_oidc_mock.well_known.contents = {'key': 'value'}
        self.setup_mock('keycloak.realm.KeycloakRealm.open_id_connect',
                        return_value=keycloak_oidc_mock)

    def test_refresh_well_known_oidc(self):
        """
        Case: An update is requested for the .well-known for a specified realm.
        Expected: The .well-known is updated.
        """
        self.assertEqual(self.realm._well_known_oidc, 'empty')

        django_keycloak.services.realm.refresh_well_known_oidc(
            realm=self.realm
        )

        self.assertEqual(self.realm._well_known_oidc, '{"key": "value"}')



import logging

from django.conf import settings
from django.contrib.auth import get_user_model
from django.core.exceptions import ImproperlyConfigured, ObjectDoesNotExist
from django.utils import timezone
from jose.exceptions import (
    ExpiredSignatureError,
    JWTClaimsError,
    JWTError,
)
from keycloak.exceptions import KeycloakClientError

import django_keycloak.services.oidc_profile


logger = logging.getLogger(__name__)


class KeycloakAuthorizationBase(object):

    def get_user(self, user_id):
        UserModel = get_user_model()

        try:
            user = UserModel.objects.select_related('oidc_profile__realm').get(
                pk=user_id)
        except UserModel.DoesNotExist:
            return None

        if user.oidc_profile.refresh_expires_before > timezone.now():
            return user

        return None

    def get_all_permissions(self, user_obj, obj=None):
        if not user_obj.is_active or user_obj.is_anonymous or obj is not None:
            return set()
        if not hasattr(user_obj, '_keycloak_perm_cache'):
            user_obj._keycloak_perm_cache = self.get_keycloak_permissions(
                user_obj=user_obj)
        return user_obj._keycloak_perm_cache

    def get_keycloak_permissions(self, user_obj):
        try:
            oidc_profile = user_obj.oidc_profile
        except ObjectDoesNotExist:
            return set()

        rpt_decoded = django_keycloak.services.oidc_profile\
            .get_entitlement(oidc_profile=oidc_profile)

        if settings.KEYCLOAK_PERMISSIONS_METHOD == 'role':
            return [
                role for role in rpt_decoded['resource_access'].get(
                    oidc_profile.realm.client.client_id,
                    {'roles': []}
                )['roles']
            ]
        elif settings.KEYCLOAK_PERMISSIONS_METHOD == 'resource':
            permissions = []
            for p in rpt_decoded['authorization'].get('permissions', []):
                if 'scopes' in p:
                    for scope in p['scopes']:
                        if '.' in p['resource_set_name']:
                            app, model = p['resource_set_name'].split('.', 1)
                            permissions.append('{app}.{scope}_{model}'.format(
                                app=app,
                                scope=scope,
                                model=model
                            ))
                        else:
                            permissions.append('{scope}_{resource}'.format(
                                scope=scope,
                                resource=p['resource_set_name']
                            ))
                else:
                    permissions.append(p['resource_set_name'])

            return permissions
        else:
            raise ImproperlyConfigured(
                'Unsupported permission method configured for '
                'Keycloak: {}'.format(settings.KEYCLOAK_PERMISSIONS_METHOD)
            )

    def has_perm(self, user_obj, perm, obj=None):

        if not user_obj.is_active:
            return False

        granted_perms = self.get_all_permissions(user_obj, obj)
        return perm in granted_perms


class KeycloakAuthorizationCodeBackend(KeycloakAuthorizationBase):

    def authenticate(self, request, code, redirect_uri):

        if not hasattr(request, 'realm'):
            raise ImproperlyConfigured(
                'Add BaseKeycloakMiddleware to middlewares')

        keycloak_openid_profile = django_keycloak.services\
            .oidc_profile.update_or_create_from_code(
                client=request.realm.client,
                code=code,
                redirect_uri=redirect_uri
            )

        return keycloak_openid_profile.user


class KeycloakPasswordCredentialsBackend(KeycloakAuthorizationBase):

    def authenticate(self, request, username, password):

        if not hasattr(request, 'realm'):
            raise ImproperlyConfigured(
                'Add BaseKeycloakMiddleware to middlewares')

        if not request.realm:
            # If request.realm does exist, but it is filled with None, we
            # can't authenticate using Keycloak
            return None

        try:
            keycloak_openid_profile = django_keycloak.services\
                .oidc_profile.update_or_create_from_password_credentials(
                    client=request.realm.client,
                    username=username,
                    password=password
                )
        except KeycloakClientError:
            logger.debug('KeycloakPasswordCredentialsBackend: failed to '
                         'authenticate.')
        else:
            return keycloak_openid_profile.user

        return None


class KeycloakIDTokenAuthorizationBackend(KeycloakAuthorizationBase):

    def authenticate(self, request, access_token):

        if not hasattr(request, 'realm'):
            raise ImproperlyConfigured(
                'Add BaseKeycloakMiddleware to middlewares')

        try:
            oidc_profile = django_keycloak.services.oidc_profile\
                .get_or_create_from_id_token(
                    client=request.realm.client,
                    id_token=access_token
                )
        except ExpiredSignatureError:
            # If the signature has expired.
            logger.debug('KeycloakBearerAuthorizationBackend: failed to '
                         'authenticate due to an expired access token.')
        except JWTClaimsError as e:
            logger.debug('KeycloakBearerAuthorizationBackend: failed to '
                         'authenticate due to failing claim checks: "%s"'
                         % str(e))
        except JWTError:
            # The signature is invalid in any way.
            logger.debug('KeycloakBearerAuthorizationBackend: failed to '
                         'authenticate due to a malformed access token.')
        else:
            return oidc_profile.user

        return None



from django.http.response import HttpResponse


class HttpResponseNotAuthorized(HttpResponse):
    status_code = 401

    def __init__(self, content=b'', authorization_method='Bearer',
                 attributes=None, *args, **kwargs):
        super(HttpResponseNotAuthorized, self).__init__(content, *args,
                                                        **kwargs)

        attributes = attributes or {}
        attributes_str = ', '.join(
            [
               '{}="{}"'.format(key, value)
               for key, value in attributes.items()
            ]
        )

        self['WWW-Authenticate'] = '{} {}'.format(authorization_method,
                                                  attributes_str)



from django.contrib import auth
from django.core.exceptions import PermissionDenied

from django_keycloak.models import RemoteUserOpenIdConnectProfile


class KeycloakRemoteUser(object):
    """
    A class based on django.contrib.auth.models.User.
    See https://docs.djangoproject.com/en/2.0/ref/contrib/auth/
    #django.contrib.auth.models.User
    """

    username = ''
    first_name = ''
    last_name = ''
    email = ''
    password = ''
    groups = []
    user_permissions = []

    _last_login = None

    def __init__(self, userinfo):
        """
        Create KeycloakRemoteUser from userinfo and oidc_profile.
        :param dict userinfo: the userinfo as retrieved from the OIDC provider
        """
        self.username = userinfo.get('preferred_username') or userinfo['sub']
        self.email = userinfo.get('email', '')
        self.first_name = userinfo.get('given_name', '')
        self.last_name = userinfo.get('family_name', '')
        self.sub = userinfo['sub']

    def __str__(self):
        return self.username

    @property
    def pk(self):
        """
        Since the BaseAbstractUser is a model, every instance needs a primary
        key. The Django authentication backend requires this.
        """
        return 0

    @property
    def identifier(self):
        """
        Identifier used for session storage.
        :rtype: str
        """
        return self.sub

    @property
    def is_staff(self):
        """
        :rtype: bool
        :return: whether the user is a staff member or not, defaults to False
        """
        return False

    @property
    def is_active(self):
        """
        :rtype: bool
        :return: whether the user is active or not, defaults to True
        """
        return True

    @property
    def is_superuser(self):
        """
        :rtype: bool
        :return: whether the user is a superuser or not, defaults to False
        """
        return False

    @property
    def last_login(self):
        """
        :rtype: Datetime
        :return: the date and time of the last login
        """
        return self._last_login

    @last_login.setter
    def last_login(self, content):
        """
        :param datetime content:
        """
        self._last_login = content

    @property
    def is_authenticated(self):
        """
        Read-only attribute which is always True.
        See https://docs.djangoproject.com/en/2.0/ref/contrib/auth/
        #django.contrib.auth.models.User.is_authenticated
        :return:
        """
        return True

    @property
    def is_anonymous(self):
        """
        Read-only attribute which is always False.
        See https://docs.djangoproject.com/en/2.0/ref/contrib/auth/
        #django.contrib.auth.models.User.is_anonymous
        :return:
        """
        return False

    def get_username(self):
        """
        Get the username
        :return: username
        """
        return self.username

    def get_full_name(self):
        """
        Get the full name (first name + last name) of the user.
        :return: the first name and last name of the user
        """
        return "{first} {last}".format(first=self.first_name,
                                       last=self.last_name)

    def get_short_name(self):
        """
        Get the first name of the user.
        :return: first name
        """
        return self.first_name

    def get_group_permissions(self, obj=None):
        pass

    def get_all_permissions(self, obj=None):
        """
        Logic from django.contrib.auth.models._user_get_all_permissions
        :param perm:
        :param obj:
        :return:
        """
        permissions = set()
        for backend in auth.get_backends():
            # Excluding Django.contrib.auth backends since they are not
            # compatible with non-db-backed permissions.
            if hasattr(backend, "get_all_permissions") \
                    and not backend.__module__.startswith('django.'):
                permissions.update(backend.get_all_permissions(self, obj))
        return permissions

    @property
    def oidc_profile(self):
        """
        Get the related OIDC Profile for this user.
        :rtype: django_keycloak.models.RemoteUserOpenIdConnectProfile
        :return: OpenID Connect Profile
        """
        try:
            return RemoteUserOpenIdConnectProfile.objects.get(sub=self.sub)
        except RemoteUserOpenIdConnectProfile.DoesNotExist:
            return None

    def has_perm(self, perm, obj=None):
        """
        Logic from django.contrib.auth.models._user_has_perm
        :param perm:
        :param obj:
        :return:
        """
        for backend in auth.get_backends():
            if not hasattr(backend, 'has_perm') \
                    or backend.__module__.startswith('django.contrib.auth'):
                continue
            try:
                if backend.has_perm(self, perm, obj):
                    return True
            except PermissionDenied:
                return False
        return False

    def has_perms(self, perm_list, obj=None):
        return all(self.has_perm(perm, obj) for perm in perm_list)

    def has_module_perms(self, module):
        """
        Logic from django.contrib.auth.models._user_has_module_perms
        :param module:
        :return:
        """
        for backend in auth.get_backends():
            if not hasattr(backend, 'has_module_perms'):
                continue
            try:
                if backend.has_module_perms(self, module):
                    return True
            except PermissionDenied:
                return False
        return False

    def email_user(self, subject, message, from_email=None, **kwargs):
        raise NotImplementedError('This feature is not implemented by default,'
                                  ' extend this class to implement')

    def save(self):
        """
        Normally implemented by django.db.models.Model
        :raises NotImplementedError: to remind that this is not a
        database-backed model and should not be used like one
        """
        raise NotImplementedError('This is not a database model')



from django.test import TestCase, override_settings

from django_keycloak.factories import OpenIdConnectProfileFactory
from django_keycloak.tests.mixins import MockTestCaseMixin
from django_keycloak.auth.backends import KeycloakAuthorizationBase


@override_settings(KEYCLOAK_PERMISSIONS_METHOD='resource')
class BackendsKeycloakAuthorizationBaseHasPermTestCase(
        MockTestCaseMixin, TestCase):

    def setUp(self):
        self.backend = KeycloakAuthorizationBase()

        self.profile = OpenIdConnectProfileFactory(user__is_active=True)

        self.setup_mock(
            'django_keycloak.services.oidc_profile.get_entitlement',
            return_value={
                'authorization': {
                    'permissions': [
                        {
                            'resource_set_name': 'Resource',
                            'scopes': [
                                'Read',
                                'Update'
                            ]
                        },
                        {
                            'resource_set_name': 'Resource2'
                        }
                    ]
                }
            }
        )

    def test_resource_scope_should_have_permission(self):
        """
        Case: Permission is expected that is available to the user.
        Expected: Permission granted.
        """
        permission = self.backend.has_perm(
            user_obj=self.profile.user, perm='Read_Resource')

        self.assertTrue(permission)

    def test_resource_no_scope_should_not_have_permission(self):
        """"
        Case: Permission is formatted as resource only which does not exist as
        such in the RPT.
        Expected: Permission denied.
        """
        permission = self.backend.has_perm(
            user_obj=self.profile.user, perm='Resource')

        self.assertFalse(permission)

    def test_resource_other_scope_should_not_have_permission(self):
        """"
        Case: Permission is expected with a scope that is not available to
        the user according to the RPT.
        Expected: Permission denied.
        """
        permission = self.backend.has_perm(
            user_obj=self.profile.user, perm='Create_Resource')

        self.assertFalse(permission)

    def test_other_resource_other_scope_should_not_have_permission(self):
        """"
        Case: Permission is expected that is not available to the user
        according to the RPT.
        Expected: Permission denied.
        """
        permission = self.backend.has_perm(
            user_obj=self.profile.user, perm='OtherScope_OtherResource')

        self.assertFalse(permission)

    def test_resource_no_scope_should_have_permission(self):
        """"
        Case: Permission is expected with no scope provided, but scope is
        also not provided in the RPT.
        Expected: Permission granted.
        """
        permission = self.backend.has_perm(
            user_obj=self.profile.user, perm='Resource2')

        self.assertTrue(permission)
